@@ -28,21 +28,20 @@
 #define REFRESH_INTERVAL     1000
 
 // File systems not supporting mod-time, refresh every nth time
-#define REFRESH_FREQUENCY    32
+#define REFRESH_FREQUENCY    30
 
 // Time interval before expanding a folder (ms)
 #define EXPAND_INTERVAL		500
 
 // Global variables
 #if defined(linux)
-#define FSTAB_PATH "/etc/fstab"
-#define MTAB_PATH "/proc/mounts"
 extern FXStringDict* fsdevices;
 extern FXStringDict* mtdevices;
 extern FXStringDict* updevices;
 #endif
 
 extern FXbool allowPopupScroll;
+extern FXString xdgdatahome;
 
 
 // Object implementation
@@ -195,12 +194,9 @@ DirList::DirList(FXWindow *focuswin,FXComposite *p,FXObject* tgt,FXSelector sel,
     }
 #endif
 
-	// Trahscan location (without trailing slash)
-	FXString homelocation=getenv("HOME");
-    if(homelocation=="")
-        homelocation=ROOTDIR;
-	trashfileslocation=homelocation+PATHSEPSTRING+TRASHFILESPATH;
-	trashinfolocation=homelocation+PATHSEPSTRING+TRASHINFOPATH;
+	// Trahscan location
+	trashfileslocation=xdgdatahome + PATHSEPSTRING TRASHFILESPATH;
+	trashinfolocation=xdgdatahome + PATHSEPSTRING TRASHINFOPATH;
 }
 
 
@@ -427,7 +423,7 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
 	FXuchar *data;
     FXuint len;
     FXbool showdialog=TRUE;
-	FXbool ret;
+	FXint ret;
     File *f=NULL;
 
     FXbool ask_before_copy=getApp()->reg().readUnsignedEntry("OPTIONS","ask_before_copy",TRUE);
@@ -454,6 +450,10 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
 			ctrlshiftkey=TRUE;
 	}
 
+    // Get DND data
+    // This is done before displaying the popup menu to fix a drag and drop problem with konqueror and dolphin file managers
+    FXbool dnd=getDNDData(FROM_DRAGNDROP,urilistType,data,len);
+
 	// Display the dnd dialog if the control or shift key were not pressed
 	if (confirm_dnd & !ctrlshiftkey)
 	{
@@ -484,7 +484,8 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
     }
 
     // Get uri-list of files being dropped
-    if(getDNDData(FROM_DRAGNDROP,urilistType,data,len))
+    //if(getDNDData(FROM_DRAGNDROP,urilistType,data,len))
+    if(dnd)  // See comment upper
     {
         FXRESIZE(&data,FXuchar,len+1);
         data[len]='\0';
@@ -509,7 +510,10 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
         else if (dropaction==DRAG_LINK)
 			f=new File(this,_("File symlink"),SYMLINK);
 		else
+		{
+	        FXFREE(&data);
             return 0;
+		}
 
         // Target directory
 		FXString targetdir=dropdirectory;
@@ -522,7 +526,7 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
             FXString source(FXURL::fileFromURL(url));
             FXString target(targetdir+PATHSEPSTRING+FXPath::name(source));
             FXString sourcedir=FXPath::directory(source);
-
+ 
             // File operation dialog, if needed
             if ((!confirm_dnd | ctrlshiftkey) & ask_before_copy & showdialog)
             {
@@ -593,7 +597,10 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
 
 				// Move source to target
 				else
+				{
+					//target=FXPath::directory(target);
 					ret=f->move(source,target);
+				}
 
 				// If source file is located at trash location, try to also remove the corresponding trashinfo if it exists
 				// Do it silently and don't report any error if it fails
@@ -618,6 +625,9 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
                     MessageBox::error(this,BOX_OK,_("Error"),_("Move file operation cancelled!"));
 					break;
                 }
+
+				// Set directory to the source parent
+				setDirectory(sourcedir,FALSE);
             }
             // Copy the source file
             else if(dropaction==DRAG_COPY)
@@ -646,7 +656,10 @@ long DirList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr)
 
 				// Copy source to target
 				else
+				{
+					//target=FXPath::directory(target);
 					ret=f->copy(source,target);
+				}
 
 				// An error has occurred
 				if (ret==0 && !f->isCancelled())
@@ -758,6 +771,7 @@ long DirList::onEndDrag(FXObject* sender,FXSelector sel,void* ptr)
         return 1;
     endDrag((didAccept()!=DRAG_REJECT));
     setDragCursor(getDefaultCursor());
+
     return 1;
 }
 
@@ -1218,37 +1232,50 @@ void DirList::scan(FXbool force)
             // Get the full path of the item
             pathname=getItemPathname((TreeItem*)item);
 
-            // Stat this directory; should not fail as parent has been scanned already
-			statrep(pathname.text(),&info);
+            // Stat this directory
+			if (statrep(pathname.text(),&info)==0)
+			{
+				// Get the mod date of the item
+				FXTime newdate=(FXTime)FXMAX(info.st_mtime,info.st_ctime);
 
-            // Get the mod date of the item
-            FXTime newdate=(FXTime)FXMAX(info.st_mtime,info.st_ctime);
+				// Forced, date was changed, or failed to get proper date and counter expired
+				if(force || (item->date!=newdate) || (counter==0))
+				{
+					// And do the refresh
+					listChildItems(item);
+					sortChildItems(item);
 
-            // Forced, date was changed, or failed to get proper date and counter expired
-            if(force || (item->date!=newdate) || (counter==0))
-            {
-                // And do the refresh
-                listChildItems(item);
-                sortChildItems(item);
+					// Remember when we did this
+					item->date=newdate;
+				}
 
-                // Remember when we did this
-                item->date=newdate;
-            }
-
-            // Go deeper
-            if(item->first)
-            {
-                item=(DirItem*)item->first;
-                continue;
-            }
+				// Go deeper
+				if(item->first)
+				{
+					item=(DirItem*)item->first;
+					continue;
+				}
+			}
+			
+			// Directory does not exist
+			else
+			{
+				// Go to parent and rescan
+				setDirectory(FXPath::directory(pathname),FALSE);
+				scan(TRUE);
+				break;
+			}	
         }
 
         // Go up
         while(!item->next && item->parent)
+        {
             item=(DirItem*)item->parent;
+		}
 
         // Go to next
         item=(DirItem*)item->next;
+        
     }
 }
 
@@ -1397,62 +1424,52 @@ fnd:
 
             // Item flags
             if(info.st_mode&(S_IXUSR|S_IXGRP|S_IXOTH))
-            {
                 item->state|=DirItem::EXECUTABLE;
-            }
             else
-            {
                 item->state&=~DirItem::EXECUTABLE;
-            }
+
             if(S_ISDIR(info.st_mode))
             {
                 item->state|=DirItem::FOLDER;
                 item->state&=~DirItem::EXECUTABLE;
             }
             else
-            {
                 item->state&=~(DirItem::FOLDER|DirItem::HASITEMS);
-            }
+
             if(S_ISCHR(info.st_mode))
             {
                 item->state|=DirItem::CHARDEV;
                 item->state&=~DirItem::EXECUTABLE;
             }
             else
-            {
                 item->state&=~DirItem::CHARDEV;
-            }
+
             if(S_ISBLK(info.st_mode))
             {
                 item->state|=DirItem::BLOCKDEV;
                 item->state&=~DirItem::EXECUTABLE;
             }
             else
-            {
                 item->state&=~DirItem::BLOCKDEV;
-            }
+
             if(S_ISFIFO(info.st_mode))
             {
                 item->state|=DirItem::FIFO;
                 item->state&=~DirItem::EXECUTABLE;
             }
             else
-            {
                 item->state&=~DirItem::FIFO;
-            }
+
             if(S_ISSOCK(info.st_mode))
             {
                 item->state|=DirItem::SOCK;
                 item->state&=~DirItem::EXECUTABLE;
             }
             else
-            {
                 item->state&=~DirItem::SOCK;
-            }
+
             if(islink)
-            {
                 item->state|=DirItem::SYMLINK;
-            }
             else
             {
                 item->state&=~DirItem::SYMLINK;
@@ -1626,8 +1643,8 @@ fnd:
     for(item=oldlist; item; item=link)
     {
         link=item->link;
-        removeItem(item,TRUE);
 		FXFREE(&item->data);
+        removeItem(item,TRUE);
     }
 
     // Now we know for sure whether we really have subitems or not
